% Arezki, Ramey, Sheng Model, "News Shocks in Open Economies: Evidence from Giant Oil Discoveries" forthcoming QJE


% One Sector ARS model, GHH preferences

% Feeds in endogenous reserves process from baseline model as an exogenous TFP-type process

% Requires: resdepletionghh.xlsx


% This program runs using Dynare 4.3.2, but not necessarily on slightly different versions.

% VARIABLE DEFINITIONS

%   y = output 
%   c = consumption
%   i = investment
%   di = i - i(-1)
%   k = capital stock
%   n = employment
%   dn = n - n(-1)
%   r = interest rate
%   m = expression in parenthesis in utility function
%   b = net foreign assets 
%   tb = trade balance 
%   ca = current account 
%   a =  TFP
%   lambda = Lagrange multiplier on net foreign asset accumulation equation
%   eta = Lagrange multiplier on capital accumulation equation, miu in NJ program

%  ENDOGENOUS VARIABLES 

var y, c, i, di, k, n, dn, m, b, r, tb, ca, sv, cy, iy, tby, cay, svy, lambda, eta, a, res; 

%  EXOGENOUS VARIABLES

varexo e;  


% PARAMETERS

parameters alpha, beta, theta, psi, sigma, gamma, gammai, delta, gamman, chi, bstar, rho, alphar; // structural parameters
parameters a_ss, res_ss, ntarget, tby_ss, k_ss, y_ss, w_ss,
  b_ss, c_ss, m_ss;  // steady-state targets


% PARAMETER VALUES AND DEFINITIONS

alpha = 0.64;  // exponent on labor in production function
beta = 1/1.06;  //Discount factor. 
theta = 1.2;  // exponent on labor in utility function, governs interteporal elasticity of substitution
sigma = 1;  //governs intertemporal substitution of consumption-labor bundle in utility function, sigma = 1 implies log
gamma = 0; //JR preference parameter.  gamma = 1 implies KPR preferences, gamma = 0 implies GHH preferences
gammai = 0.1; // 2nd derivative of investment adjustment cost function
delta = 0.1;  //  10% is standard for annual capital depreciation rate, JR's is quarterly = 0.0125, annual = 5%
gamman = 0;  // 2nd derivative of labor adjustment cost function
chi = 0.0001;  // parameter on elasticity of interest rate with respect to net foreign assets.
ntarget=20;
a_ss = 0.524;
res_ss = 6.032338246; //Steady-state value of reserves for endogenous depletion model
tby_ss = 0.04;
rho = 0.7;


alphar = 1 - 0.13 - 0.49;

%%%%% SOLVED STEADY-STATE CAPITAL-LABOR RATIOS

kappa = ((1/beta - 1 + delta)/(res_ss^alphar*a_ss*(1-alpha)))^(1/(-alpha)); // SS capital-labor ratio 
w_ss = alpha*res_ss^alphar*a_ss*kappa^(1-alpha);

%%%% OTHER RELATIONSHIPS AND STEADY-STATE GUESSES

k_ss = ntarget*kappa;

y_ss = res_ss^alphar*a_ss*ntarget^alpha*k_ss^(1-alpha);

c_ss = (1 - tby_ss - delta*k_ss/y_ss)*y_ss;

bstar = -tby_ss*y_ss/(1/beta - 1);  //parameter in interest rate function, set to match tby_ss

psi = w_ss/(theta*ntarget^(theta-1)); // parameter to target steady-state n

tb_ss = tby_ss*y_ss;
b_ss = bstar;
m_ss = c_ss - psi*ntarget^theta;
lambda_ss = (c_ss - psi*ntarget^theta)^(-sigma);


model;  


% 1) DEFINITION OF m (variable in parenthesis of utility function)

                 m = (c-psi*n^theta);                                                   

% 2) FOC C
            m^(-sigma) = lambda;

% 3) DEFINITION OF DN

              dn = n/n(-1);
% 4) FOC N
             m^(-sigma)*psi*theta*n^(theta-1)
             = alpha*lambda*y/n
             - lambda*(gamman/2)*(dn-1)^2
             - lambda*dn*gamman*(dn-1)
             + beta*lambda(+1)*(dn(+1))^2*gamman*(dn(+1)-1) ;
 

% 5) K LAW OF MOTION

            k = (1-delta)*k(-1)+ i*(1 - (gammai/2)*(di-1)^2);
             
% 6) FOC K
              eta = beta*eta(+1)*(1-delta)
             + beta*lambda(+1)*(1-alpha)*y(+1)/k;


% 7) NET FOREIGN ASSETS LAW OF MOTION

            b = (1 + r)*b(-1) + tb;  


% 8)  FOC NET FOREIGN ASSETS B (B for bonds)

      lambda = beta*lambda(+1)*(1 + r(+1)); 


% 9) DEFINITION OF DI

          di = i/i(-1);

% 10) FOC I

           lambda = eta*(1 - (gammai/2)*(di-1)^2)
           - eta*di*gammai*(di-1)
           + beta*eta(+1)*di(+1)^2*gammai*(di(+1)-1);    

% 11) RELATION OF R TO NET FOREIGN ASSETS

      r = 1/beta - 1 + chi*( exp( bstar - b(-1)) - 1); 

% 12) DEFINITION OF OUTPUT

          y = a(-1)*n^alpha*(k(-1)^(1-alpha))*res(-1)^alphar; 

% 15) and 16) TRADE BALANCE DEFINITIONS

          tb = y - c - i - n*(gamman/2)*(dn-1)^2 ;

          tby = tb/y;  

% 17) and 18) CURRENT ACCOUNT DEFINITIONS

         ca = tb + r*b(-1);  // JR timing
          cay = ca/y; // ratio to output 

% 19) Consumption ratio

       cy = c/y;  

% 20) Investment ratio

        iy = i/y;   

% 21) & 22) Saving Definitions

         sv = ca + i;
          svy = sv/y;  //  the irf for svy is the same as the above one. 

   
% 23) TFP and reserves

   a = a_ss;
   res = res_ss + 0.049*e; // 0.06 to match S-S y2/y, use 0.049 to match peak y

end;

initval; 
y            =     		 y_ss;
c            =     		 c_ss;
i            =     		 0.1*k_ss;
di           =     		 1;
k            =     		 k_ss;
n            =     		 ntarget;
dn           =     		 1;
m            =    		 m_ss;
b            =     		 bstar;
r            =     		 0.06;
tb           =     		 0.04*y_ss;
ca           =     		 0;
sv           =     		 y_ss - c_ss;
cy            =    		 c_ss/y_ss;
iy           =     		 0.1*k_ss/y_ss;
tby          =     		 0.04;
cay         =      		 0;
svy          =     		 1 - c_ss/y_ss;
lambda      =      		 m_ss^(-sigma);
eta         =      		 m_ss^(-sigma);
a          =       		 a_ss;
res         =           res_ss;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FOR STOCHASTIC SIMULTIONS
% FOR NEWS, USE THE RES EQUATION ABOVE WITH E(-5)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

check;
steady;

resfile = 'resdepletionghh.xlsx';
resdep = xlsread(resfile);

shocks;
var e;
periods 1:100;
values (resdep);  //Deviation of endogenous reserves from S-S, from arsdepletionghh
end;

simul(periods=100);

%% To print out results (with lots of significant figures) type the following in matlab
%% A=[oo_.exo_simul, oo_.endo_simul']
%% Then double-click A matrix in upper right to get it to print it with significant figures
%%  and then copy to Excel